package compiler

import "fmt"

// Filetype represents an output filetype.
type Filetype int; const (
	FiletypeUnknown Filetype = iota
	FiletypeObject
	FiletypeLibrary
	FiletypeAssembly
	FiletypeIR
)

// FiletypeFromString returns a filetype based on the given name.
func FiletypeFromString (ext string) (Filetype, bool) {
	switch ext {
	case "":    return FiletypeUnknown,  true
	case "obj": return FiletypeObject,   true
	case "lib": return FiletypeLibrary,  true
	case "asm": return FiletypeAssembly, true
	case "ir":  return FiletypeIR,       true
	default:    return FiletypeUnknown,  false
	}
}

// FiletypeFromExt returns a filetype based on the given filename extension.
func FiletypeFromExt (ext string) (Filetype, bool) {
	// FIXME some of these are platform dependent, for example on windows
	// FiletypeLib would be .dll
	switch ext {
	case ".o":  return FiletypeObject,   true
	case ".so": return FiletypeLibrary,  true
	case ".s":  return FiletypeAssembly, true
	case ".ll": return FiletypeIR,       true
	default:    return FiletypeUnknown,  false
	}
}

// String returns a string representation of the filetype.
func (filetype Filetype) String () string {
	switch filetype {
	case FiletypeUnknown:  return ""
	case FiletypeObject:   return "obj"
	case FiletypeLibrary:  return "lib"
	case FiletypeAssembly: return "asm"
	case FiletypeIR:       return "ir"
	default:               return fmt.Sprintf("Filetype(%d)", filetype)
	}
}

// Ext returns the standard filename extension of the filetype.
func (filetype Filetype) Ext () string {
	// FIXME again, some of these are platform dependent
	switch filetype {
	case FiletypeUnknown:  return ""
	case FiletypeObject:   return ".o"
	case FiletypeLibrary:  return ".so"
	case FiletypeAssembly: return ".s"
	case FiletypeIR:       return ".ll"
	default:               return ""
	}
}

// Extend adds the extension of the filetype onto the specified path.
func (filetype Filetype) Extend (path string) string {
	return path + filetype.Ext()
}
